/*
 * This file is part of activationd.
 *
 * Copyright © 2019 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _DATABASE_H_
#define _DATABASE_H_

#include "common.h"
#include "module.h"
#include "event.h"

#define EPC_LAST_BOOT_OID "org.tizen.epc.last_boot_oid"

struct epc_database_adapter {
	struct epc_module module;
	char *name;
	const char *id_key;

	/**
	 * @param[out] oid Will be filled with newly created oid
	 */
	int (*store)(struct epc_database_adapter *adapter,
				 struct epc_object *obj, epc_oid_t *oid);

	int (*get_by_oid)(struct epc_database_adapter *adapter,
					  epc_oid_t *oid, struct epc_object *result);

	int (*load)(struct epc_database_adapter *adapter,
				struct epc_object *query, struct epc_object *hints,
				struct epc_object *result, uint32_t *nr);

	int (*get_well_known_oid)(const char *name, epc_oid_t *oid);

	bool (*is_oid_valid)(epc_oid_t *oid);

	int (*get_event)(epc_oid_t *oid, struct epc_event **ev);
};

#define to_database_adapter(MOD)				\
	container_of(MOD, struct epc_database_adapter, module)

struct epc_database_adapter *epc_get_database_adapter(void);
int epc_set_database_adapter(struct epc_database_adapter *adapter);
int database_store(struct epc_object *obj, epc_oid_t *oid);
int database_get_by_oid(epc_oid_t *oid, struct epc_object *result);
int database_load(struct epc_object *query, struct epc_object *hints,
				  struct epc_object *result, uint32_t *nr);
int database_get_well_known_oid(const char *name, epc_oid_t *oid);
bool database_is_oid_valid(epc_oid_t *oid);
const char *database_get_id_key(void);
int database_get_event(epc_oid_t *oid, struct epc_event **ev);
int database_dump(struct epc_object **out);

#endif	/* _DATABASE_H_ */
