/*
 * This file is part of activationd.
 *
 * Copyright © 2019 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _EPC_DBUS_SIGNAL_EVENT_H
#define _EPC_DBUS_SIGNAL_EVENT_H

#include <time.h>
#include <glib.h>
#include "event.h"

#define DBUS_SIGNAL_EVENT_ID "dbus_signal"
#define DS_EV_TIME "et"
#define DS_EV_ID "event_id"
#define DS_EV_SENDER "sender"
#define DS_EV_INTERFACE "interface"
#define DS_EV_MEMBER "member"
#define DS_EV_PATH "path"
#define DS_EV_PATH_NAMESPACE "path_namespace"
#define DS_EV_DESTINATION "destination"
#define DS_EV_PARAMS "params"

struct dbus_signal_event {
	struct epc_event event;
	time_t event_time;
	char *event_id;
	char *sender;
	char *interface;
	char *member;
	char *path;
	char *path_namespace;
	char *destination;
	struct epc_object *match_params;
	GVariant *signal_params;
};

struct dbus_signal_event_data {
	time_t event_time;
	char *event_id;
	char *sender;
	char *interface;
	char *member;
	char *path;
	char *path_namespace;
	char *destination;
	struct epc_object *match_params;
	GVariant *signal_params;
};

#define to_dbus_signal_event(EVENT)						\
	container_of(EVENT, struct dbus_signal_event, event)

#endif /* _EPC_DBUS_SIGNAL_EVENT_H */
