/*
 * This file is part of activationd.
 *
 * Copyright © 2019 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef EPC_DECISION_MADE_EVENT_H
#define EPC_DECISION_MADE_EVENT_H

#include <time.h>

#include "action.h"
#include "event.h"
#include "common.h"

#define DECISION_MADE_EVENT_ID "decision_made"
#define DM_EV_WHO "who"
#define DM_EV_ACTION "act"
#define DM_EV_ACTION_DATA "actd"
#define DM_EV_REASON "rsn"

struct decision_made_event;

typedef void (*on_action_finished_cb)(struct decision_made_event *ev, int result);

struct decision_made_event {
	struct epc_event event;
	struct epc_event *reason;
	char *who_made;
	char *action;
	on_action_finished_cb on_finished;
	struct epc_object *action_data;
	/* TODO: what more do we need? */
};

struct dm_event_data {
	/* This is referenced by dm event */
	struct epc_event *reason;
	/* Strings are deep copied */
	char *who_made;
	char *action;
	on_action_finished_cb on_finished;

	struct epc_object *action_data;
};

#define to_decision_made_event(EVENT)						\
	container_of(EVENT, struct decision_made_event, event)

#endif /* EPC_DECISION_MADE_EVENT_H */
