/*
 * This file is part of activationd.
 *
 * Copyright © 2019 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef EPC_EVENT_PROCESSOR_H
#define EPC_EVENT_PROCESSOR_H

#include "event.h"
#include "notify_queue.h"
#include "list.h"

struct epc_event_handler {
	char *name;

	int (*init)(struct epc_event_handler *handler, struct epc_config *config);
	void (*cleanup)(struct epc_event_handler *handler);

	/* Would you like to handle this? */
	int (*event_match)(struct epc_event_handler *handler,
					   struct epc_event *ev);

	/* Called each time when there is sth in a queue */
	int (*handle_event)(struct epc_event_handler *handler);

	/* Initialized internally by FW */
	struct nqueue_head event_queue;
	struct sd_event_source *event_queue_sd_source;

	/* has to be initialized */
	struct list_head node;
};

int event_processor_report_event(struct epc_event *ev);

int event_processor_handler_register(struct epc_event_handler *handler);
void event_processor_handler_unregister(struct epc_event_handler *handler);

#define EPC_EVENT_HANDLER_REGISTER(EVENT_HANDLER, NAME, TYPE)	    \
static int event_handler_ ##NAME## _init(struct epc_module *module,  \
										 struct epc_config *config,	\
										 sd_event *event_loop)			\
{																		\
	int ret;															\
																		\
	if (EVENT_HANDLER.init) {											\
		ret = EVENT_HANDLER.init(&EVENT_HANDLER, config);				\
		if (ret)														\
			return ret;													\
	}																	\
																		\
	return event_processor_handler_register(&EVENT_HANDLER);			\
}																		\
																		\
static void event_handler_ ##NAME## _cleanup(struct epc_module *module) \
{																		\
	if (EVENT_HANDLER.cleanup)											\
		EVENT_HANDLER.cleanup(&EVENT_HANDLER);							\
																		\
	event_processor_handler_unregister(&EVENT_HANDLER);					\
}																		\
																		\
static struct epc_module event_handler_ ##NAME## _module = {			\
	.name = #NAME,														\
	.type = TYPE,														\
	.init = event_handler_ ##NAME## _init,								\
	.cleanup = event_handler_ ##NAME## _cleanup,						\
	.node = LIST_HEAD_INIT(event_handler_ ##NAME## _module.node),		\
};																		\
																		\
EPC_MODULE_REGISTER(&event_handler_ ##NAME## _module)

#endif /* EPC_EVENT_PROCESSOR_H */
