/*
 * This file is part of activationd.
 *
 * Copyright © 2019 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef EPC_MODULE_H
#define EPC_MODULE_H

#include <time.h>
#include <systemd/sd-event.h>

#include "common.h"
#include "epc-config.h"
#include "list.h"

#ifdef TEST_BUILD

#define TEST_ENTRY_POINT_NAME(MODULE, FUNC) \
	CONCAT(test_, CONCAT(MODULE, _ ## FUNC))

#define TEST_ENTRY_POINT(func) \
	typeof(func) (*CONCAT(EPC_MODNAME_T, _ ## func)) = func

#else
#define TEST_ENTRY_POINT(func)
#endif

#define EPC_MODNAME STR(EPC_MODNAME_T)

/* Order in this enum is also order of initialization */
enum epc_module_type {
	EPC_MODULE_TYPE_MIN = 0,
	EPC_MODULE_TYPE_UNCATEGORIZED = EPC_MODULE_TYPE_MIN,
	EPC_MODULE_TYPE_CORE,
	EPC_MODULE_TYPE_DBADAPTER,
	EPC_MODULE_TYPE_EVENT,
	EPC_MODULE_TYPE_ACTION_EXECUTOR,
	EPC_MODULE_TYPE_ACTION,
	EPC_MODULE_TYPE_DECISION_MAKER,
	EPC_MODULE_TYPE_LISTENER,
	EPC_MODULE_TYPE_MAX,
};

struct epc_module {
	char *name;
	enum epc_module_type type;
	bool disabled;

	int (*init)(struct epc_module *, struct epc_config *config, sd_event *event_loop);
	void (*cleanup)(struct epc_module *);

	struct list_head node;
};

int epc_module_register(struct epc_module *module);
void epc_module_unregister(struct epc_module *module);

int epc_modules_init(sd_event *event_loop, struct epc_config *conf);
void epc_modules_cleanup(void);

struct epc_module *epc_get_module_by_name(const char *name);
void epc_disable_module_category(int category);

int epc_modules_load(void);
void epc_modules_unload(void);

#ifndef __CONSTRUCTOR__
#define __CONSTRUCTOR__ __attribute__((constructor))
#endif

#ifndef __DESTRUCTOR__
#define __DESTRUCTOR__ __attribute__((destructor))
#endif

#define EPC_MODULE_REGISTER(module)				\
	static void __CONSTRUCTOR__ module_init(void)	\
	{												\
		epc_module_register(module);				\
	}												\
	static void __DESTRUCTOR__ module_cleanup(void)	\
	{												\
		epc_module_unregister(module);			\
	}

#endif /* EPC_MODULE_H */
