/*
 * This file is part of activationd.
 *
 * Copyright © 2019 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef EPC_SERVICE_H
#define EPC_SERVICE_H

#include <sys/types.h>
#include <unistd.h>

#include "common.h"

#define SYSD_SERVICE "sd_sv"
#define SYSD_SERVICE_DBUS_PATH "db_p"
#define SYSD_SERVICE_SERVICE_TYPE "sv_t"
#define SYSD_SERVICE_RECOVERY_UNIT "rcv_u"

#define EPC_SERVICE_TYPE_VIP "org.tizen.epc.service.VIP"
#define EPC_SERVICE_TYPE_NORMAL NULL

struct systemd_service {
	char *dbus_path;
	char *service_type;
	char *recovery_unit;
};

int systemd_service_init_by_pid(pid_t pid, struct systemd_service *s);
int systemd_service_init(const char *dbus_path, struct systemd_service *s);
void systemd_service_cleanup(struct systemd_service *s);
int systemd_get_unit_by_pid(pid_t pid, const char **name);
void systemd_service_serialize(struct systemd_service *s, struct epc_object *out);
int systemd_service_deserialize(const struct epc_object *out, struct systemd_service *s);

static inline int systemd_service_is_of_type(struct systemd_service *s,
											 char *type)
{
	if (s->service_type == NULL || type == NULL)
		return s->service_type == NULL && type == NULL;

	return strcmp(s->service_type, type) == 0;
}

#endif /* EPC_SERVICE_H */
