/*
 * This file is part of activationd.
 *
 * Copyright © 2019 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef EPC_SYSTEMD_DBUS_H
#define EPC_SYSTEMD_DBUS_H

#include <systemd/sd-bus.h>
#include <libsyscommon/dbus-system.h>

#define SYSTEMD_SERVICE "org.freedesktop.systemd1"

#define SYSTEMD_OBJ "/org/freedesktop/systemd1"

#define SYSTEMD_MANAGER_INTERFACE "org.freedesktop.systemd1.Manager"
#define SYSTEMD_UNIT_INTERFACE "org.freedesktop.systemd1.Unit"

#define SYSTEMD_PRIVATE_BUS "unix:path=/run/systemd/private"
#define SYSTEMD_KERNEL_BUS "kernel:path=/sys/fs/kdbus/0-system/bus"

int epc_dbus_call(char *service, char *obj, char *interface, char *method,
					 char *args, ...);

#define epc_dbus_call_simple(SRV, OBJ, INTF, METHOD, ARGS, ...)	\
	epc_dbus_call(SRV, OBJ, INTF, METHOD, ARGS, ##__VA_ARGS__)

#define epc_dbus_call_systemd_simple(OBJ, INTF, METHOD, ARGS, ...) \
	epc_dbus_call_simple(SYSTEMD_SERVICE, OBJ, INTF, METHOD,		  \
							ARGS, ##__VA_ARGS__)

typedef dbus_pending_cb epc_dbus_handler_t;

int epc_dbus_call_async(char *service, char *obj, char *interface, char *method,
					 epc_dbus_handler_t cb,
					 void *user_data, char *args, ...);

#define epc_dbus_call_simple_async(SRV, OBJ, INTF, METHOD, CB, DATA, ARGS, ...)	\
	epc_dbus_call_async(SRV, OBJ, INTF, METHOD, CB, DATA, ARGS, ##__VA_ARGS__)

#define epc_dbus_call_systemd_simple_async(OBJ, INTF, METHOD, CB, DATA, ARGS, ...) \
	epc_dbus_call_simple_async(SYSTEMD_SERVICE, OBJ, INTF, METHOD, CB, DATA,	  \
							ARGS, ##__VA_ARGS__)

int epc_acquire_systemd_bus(sd_bus **bus);
void epc_set_systemd_private(bool);

#endif /* EPC_SYSTEMD_DBUS_H */
