/*
 * This file is part of activationd.
 *
 * Copyright © 2019 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _EPC_VCONF_KEY_CHANGED_EVENT_H
#define _EPC_VCONF_KEY_CHANGED_EVENT_H

#include <time.h>
#include "event.h"

#define VCONF_KEY_CHANGED_EVENT_ID "vconf_key_changed"
#define VKC_EV_TIME "et"
#define VKC_EV_NAME "name"
#define VKC_EV_OLDVAL "oldval"
#define VKC_EV_NEWVAL "newval"

enum vkc_type {
	VKC_BOOL,
	VKC_INT,
	VKC_DOUBLE,
	VKC_STRING
};

struct vkc_value {
	enum vkc_type type;
	union {
		bool b;
		int i;
		double d;
		char *s;
	} value;
};

struct vconf_key_changed_event {
	struct epc_event event;
	char *key_name;
	struct vkc_value oldval;
	struct vkc_value newval;
	time_t event_time;
};

struct vconf_key_changed_event_data {
	char *key_name;
	struct vkc_value oldval;
	struct vkc_value newval;
	time_t event_time;
};

#define to_vconf_key_changed_event(EVENT)						\
	container_of(EVENT, struct vconf_key_changed_event, event)

#endif /* _EPC_VCONF_KEY_CHANGED_EVENT_H */
