/*
 * This file is part of activationd.
 *
 * Copyright © 2019 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <malloc.h>

#include "action_executed_event.h"
#include "action.h"
#include "event_processor.h"
#include "log.h"

static int ae_event_match(struct epc_event_handler *handler,
						  struct epc_event *ev)
{
	return epc_event_is_of_type(ev, ACTION_EXECUTED_EVENT_ID);
}

static int ae_handle_event(struct epc_event_handler *handler)
{
	struct epc_event *ev = pop_epc_event(&handler->event_queue);

	epc_event_unref(ev);
	return 0;
}

static struct epc_event_handler action_executed_event_handler = {
	.name = "action_executed_empty_handler",
	.event_match = ae_event_match,
	.handle_event = ae_handle_event,

	.node = LIST_HEAD_INIT(action_executed_event_handler.node),
};

EPC_EVENT_HANDLER_REGISTER(action_executed_event_handler,
							  action_executed_eh,
							  EPC_MODULE_TYPE_ACTION_EXECUTOR)
