/*
 * This file is part of activationd.
 *
 * Copyright © 2019 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "action.h"
#include "action_executor.h"
#include "decision_made_event.h"
#include "unit_control_event.h"
#include "log.h"
#include "systemd_dbus.h"
#include "dbus_signal_event.h"
#include "common.h"

#include <libsyscommon/dbus-system.h>

static int forward_signal(struct epc_action *action,
		struct action_executed_event *exe_info)
{
	struct epc_event *ev = pop_epc_event(&action->execution_queue);
	struct decision_made_event *dm_ev = to_decision_made_event(ev);
	char *signal_action = NULL;
	char *signal_dest = NULL;
	char *signal_path = NULL;
	char *signal_interface = NULL;
	char *signal_member = NULL;
	GVariant *params = NULL;
	struct dbus_signal_event *ds_ev = to_dbus_signal_event(dm_ev->reason);

	exe_info->reason = ev;

	if (ds_ev)
		params = ds_ev->signal_params;

	epc_object_get_string(dm_ev->action_data, EPC_AD_SIGNAL_ACTION, &signal_action);
	epc_object_get_string(dm_ev->action_data, EPC_AD_SIGNAL_DEST, &signal_dest);
	epc_object_get_string(dm_ev->action_data, EPC_AD_SIGNAL_PATH, &signal_path);
	epc_object_get_string(dm_ev->action_data, EPC_AD_SIGNAL_INTERFACE, &signal_interface);
	epc_object_get_string(dm_ev->action_data, EPC_AD_SIGNAL_MEMBER, &signal_member);
	log_debug("forward_signal: action %s, dest %s, path %s, interface %s, member %s, params: %s",
		  signal_action, signal_dest, signal_path, signal_interface, signal_member, g_variant_print(params, TRUE));

	if (!signal_dest || !signal_path || !signal_interface || !signal_member)
		goto err;

	dbus_handle_emit_dbus_signal(signal_dest, signal_path, signal_interface, signal_member, params);

	exe_info->result = 0;
	return 0;

err:
	epc_object_append_string(exe_info->action_log, "error", "Invalid parameters given for forward_signal");
	exe_info->result = -EINVAL;
	log_error("Invalid parameters given for forward_signal");

	return 0;
}

static struct epc_action forward_signal_action = {
	.action_id = EPC_ACTION_DBUS_SIGNAL_FORWARD_ID,
	.impl_name = EPC_DEFAULT_ACTION_IMPL,
	.execute = forward_signal,
	.node = LIST_HEAD_INIT(forward_signal_action.node),
};

EPC_ACTION_REGISTER_SIMPLE(forward_signal_action);
