/*
 * This file is part of activationd.
 *
 * Copyright © 2019 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <errno.h>

#include "common.h"
#include "log.h"
#include "action.h"

int epc_fill_for_srv_restart(struct epc_object *obj, const char *service_path)
{
	int ret;

	ret = epc_object_append_string(obj, EPC_AD_SERVICE_NAME, service_path);
	if (ret < 0) {
		log_error("Unable to append %s:%s",
				  EPC_AD_SERVICE_NAME, service_path);
		return -ENOMEM;
	}

	return 0;
}

int epc_fill_for_srv_recover(struct epc_object *obj, const char *service_path, const char *recovery)
{
	int ret;

	ret = epc_object_append_string(obj, EPC_AD_SERVICE_NAME, service_path);
	if (ret < 0) {
		log_error("Unable to append %s:%s",
				  EPC_AD_SERVICE_NAME, service_path);
		return -ENOMEM;
	}

	if (recovery) {
		ret = epc_object_append_string(obj, EPC_AD_CLEANUP_UNIT, recovery);
		if (ret < 0) {
			log_error("Unable to append %s:%s",
					  EPC_AD_CLEANUP_UNIT, recovery);
			return -ENOMEM;
		}
	}

	return 0;
}

int epc_fill_for_unit_start(struct epc_object *obj, const char *unit_path)
{
	int ret;

	ret = epc_object_append_string(obj, EPC_AD_UNIT_NAME, unit_path);
	if (ret < 0) {
		log_error("Unable to append %s:%s",
				  EPC_AD_UNIT_NAME, unit_path);
		return -ENOMEM;
	}

	return 0;
}

int epc_fill_for_unit_action(struct epc_object *obj, const char *unit_action)
{
	int ret;

	ret = epc_object_append_string(obj, EPC_AD_UNIT_ACTION, unit_action);
	if (ret < 0) {
		log_error("Unable to append %s:%s",
				  EPC_AD_UNIT_ACTION, unit_action);
		return -ENOMEM;
	}

	return 0;
}

int epc_fill_for_signal_forward(struct epc_object *obj, const char *dest,
				const char *path, const char *interface,
				const char *member)
{
	assert(dest);
	assert(path);
	assert(interface);
	assert(member);

	int ret;

	ret = epc_object_append_string(obj, EPC_AD_SIGNAL_DEST, dest);
	ret += epc_object_append_string(obj, EPC_AD_SIGNAL_PATH, path);
	ret += epc_object_append_string(obj, EPC_AD_SIGNAL_INTERFACE, interface);
	ret += epc_object_append_string(obj, EPC_AD_SIGNAL_MEMBER, member);

	if (ret != 0) {
		log_error("Unable to append parameters for forward_signal");
		return -ENOMEM;
	}

	return 0;
}
