#include <errno.h>
#include <systemd/sd-id128.h>

#include "daemon_started_event.h"

static int fs_event_allocate(struct epc_event_type *type,
							 void *data, struct epc_event **ev)
{
	struct daemon_started_event *fs_ev;
	int ret;

	fs_ev = malloc(sizeof(*fs_ev));
	if (!fs_ev)
		return -ENOMEM;

	ret = epc_event_init_internal(type, &fs_ev->event);
	if (ret)
		goto free_fs_ev;

	*ev = &fs_ev->event;
	return 0;

free_fs_ev:
	free(fs_ev);
	return ret;
}

static int deserialize_fs_event(struct epc_event_type *type,
								struct epc_object *data, struct epc_event **ev)
{
	int ret = -EINVAL;

	ret = fs_event_allocate(type, NULL, ev);
	if (ret < 0)
		goto finish;

	ret = epc_event_deserialize_internal(data, type, *ev);
	if (ret < 0) {
		struct daemon_started_event *fs_ev =
			to_daemon_started_event(*ev);
		free(fs_ev);
		goto finish;
	}

	ret = 0;
finish:
	return ret;
}
TEST_ENTRY_POINT(deserialize_fs_event);

static void fs_event_release(struct epc_event *ev)
{
	struct daemon_started_event *fs_ev =
		to_daemon_started_event(ev);

	epc_event_cleanup_internal(&fs_ev->event);
	free(fs_ev);
}

static void fs_event_serialize(struct epc_event *ev, struct epc_object *out)
{
	epc_event_serialize_internal(ev, out);
}
TEST_ENTRY_POINT(fs_event_serialize);

static struct epc_event_type daemon_started_event_type = {
	.name = DAEMON_STARTED_EVENT_ID,
	.default_ops = {
		.release = fs_event_release,
		.serialize = fs_event_serialize,
	},
	.allocate_event = fs_event_allocate,
	.deserialize_event = deserialize_fs_event,
	.node = LIST_HEAD_INIT(daemon_started_event_type.node),
};

EPC_EVENT_TYPE_REGISTER(daemon_started_event_type, daemon_started_et)
