/*
 * This file is part of activationd.
 *
 * Copyright © 2019 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef ACTIVATIOND_ACCEPTANCE_TESTS_H
#define ACTIVATIOND_ACCEPTANCE_TESTS_H

#include <systemd/sd-bus.h>
#include "vconf_key_changed_event.h"

struct vkc_value;
struct test_data;

enum unit_state {
	UNKNOWN,
	ACTIVE,
	INACTIVE
};

enum {
	TEST_FAIL,
	TEST_SUCCESS,
	TEST_UNKNOWN
};

struct action {
	struct vkc_value params[3];
	enum unit_state expected_states[2];
	int param_count;
	int state_count;
	int current_state;
};

struct test_sequence {
	const char *desc;
	struct action actions[3];
	const char *id; /* vconf key or dbus method */
	int current_action;
	int expected_result;
	int result;
	char *unit;
	char *path;
	int (*step)(struct test_data *test);
};


struct test_data {
	sd_bus *bus;
	sd_event *loop;
	sd_event_source *step_timer;
	struct test_sequence *tests;
	int tests_count;
	int current_test;
	enum unit_state state;
	enum unit_state prev_state;
};

int start_test(struct test_data *data);
int on_properties_changed(sd_bus_message *m, void *userdata, sd_bus_error *ret_error);

#endif /* ACTIVATIOND_ACCEPTANCE_TESTS_H */
