/*
 * This file is part of activationd.
 *
 * Copyright © 2019 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "activationd_acceptance_tests.h"
#include "activationd_acceptance_tests_dbus_signal.h"

#define DBUS_TEST_INTERFACE "org.freedesktop.ActivationdTestInterface"
#define DBUS_TEST_PATH "/org/freedesktop/activationd/test/object"

static int send_dbus_signal(sd_bus *bus, const char *method, int param_cnt, const struct vkc_value *params)
{
	_cleanup_(sd_bus_message_unrefp) sd_bus_message *m = NULL;
	int ret;

	ret = sd_bus_message_new_signal(bus, &m, DBUS_TEST_PATH, DBUS_TEST_INTERFACE, method);
	if (ret < 0)
		goto fail;

	for (int i = 0; i < param_cnt; i++) {
		switch(params[i].type) {
		case VKC_STRING:
			ret = sd_bus_message_append(m, "s", params[i].value.s);
			if (ret < 0)
				goto fail;
			break;
		case VKC_DOUBLE:
			ret = sd_bus_message_append(m, "d", params[i].value.d);
			if (ret < 0)
				goto fail;
			break;
		case VKC_INT:
			ret = sd_bus_message_append(m, "i", params[i].value.i);
			if (ret < 0)
				goto fail;
			break;
		case VKC_BOOL:
			ret = sd_bus_message_append(m, "b", params[i].value.b);
			if (ret < 0)
				goto fail;
			break;
		}
	}
	sd_bus_send(bus, m, NULL);
	/* return value of sd_bus_send is useless because it will always fail on negative tests,
	 * always return 0 instead */
	return 0;
fail:
	return ret;
}

int dbus_signal_step(struct test_data *data)
{
	assert(data);
	struct test_sequence *test = &data->tests[data->current_test];
	return send_dbus_signal(data->bus, test->id, test->actions[test->current_action].param_count, test->actions[test->current_action].params);
}
