/*
 * This file is part of activationd.
 *
 * Copyright © 2019 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "activationd_acceptance_tests.h"
#include "activationd_acceptance_tests_unit_control.h"
#include "unit_control.h"

#define ACTD_DBUS_API_INTERFACE_NAME "org.tizen.activationd"
#define ACTD_DBUS_API_SERVICE_NAME "org.tizen.activationd"
#define ACTD_DBUS_API_OBJECT_PATH "/org/tizen/activationd"

static int unit_control_call(sd_bus *bus, const char *unit, int param_cnt, const struct vkc_value *params)
{
	assert(bus);
	assert(unit);
	assert(params);

	_cleanup_(sd_bus_message_unrefp) sd_bus_message *m = NULL;
	int ret;
	char err[512];

	if (param_cnt != 1) {
		fprintf(stderr, "invalid param count (%d)\n", param_cnt);
		ret = -EINVAL;
		goto fail;
	}

	if (params[0].type != VKC_STRING) {
		fprintf(stderr, "invalid param type\n");
		ret = -EINVAL;
		goto fail;
	}

	ret = sd_bus_message_new_method_call(
			bus,
			&m,
			ACTD_DBUS_API_SERVICE_NAME,
			ACTD_DBUS_API_OBJECT_PATH,
			ACTD_DBUS_API_INTERFACE_NAME,
			params[0].value.s);

	if (ret < 0) {
		if (strerror_r(-ret, err, sizeof err))
			fprintf(stderr, "failed to issue dbus api method call: %s", err);
		goto fail;
	}

	ret = sd_bus_message_append(m, "s", unit);
	if (ret < 0) {
		fprintf(stderr, "failed to create dbus api call\n");
		goto fail;
	}


	sd_bus_call(bus, m, 0, NULL, NULL);
	/* return value of sd_bus_call is useless because it will always fail on negative tests,
	 * always return 0 instead */
	return 0;
fail:
	return ret;
}

int unit_control_step(struct test_data *data)
{
	assert(data);
	struct test_sequence *test = &data->tests[data->current_test];
	return unit_control_call(data->bus, test->unit, test->actions[test->current_action].param_count, test->actions[test->current_action].params);
}
