/*
 * This file is part of activationd.
 *
 * Copyright © 2019 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "activationd_acceptance_tests.h"
#include "activationd_acceptance_tests_vconf.h"
#include "vconf_key_changed_event.h"
#include <vconf.h>

static int set_vconf_key(const char *key, const struct vkc_value *val)
{
	assert (key);

	keylist_t *kl = NULL;
	kl = vconf_keylist_new();
	if (!kl) {
		fprintf(stderr, "error creating vconf keylists\n");
		return -1;
	}
	switch (val->type) {
	case VKC_STRING:
		vconf_keylist_add_str(kl, key, val->value.s);
		break;
	case VKC_INT:
		vconf_keylist_add_int(kl, key, val->value.i);
		break;
	case VKC_DOUBLE:
		vconf_keylist_add_dbl(kl, key, val->value.d);
		break;
	case VKC_BOOL:
		vconf_keylist_add_bool(kl, key, val->value.b);
		break;
	default:
		vconf_keylist_free(kl);
		return -1;
	}
	if (vconf_set(kl) < 0) {
		fprintf(stderr, "error saving vconf key\n");
		vconf_keylist_free(kl);
		return -1;
	}

	vconf_keylist_free(kl);
	return 0;
}

int vconf_step(struct test_data *data)
{
	assert(data);
	struct test_sequence *test = &data->tests[data->current_test];
	return set_vconf_key(test->id, test->actions[test->current_action].params);
}
